@@ -4,7 +4,7 @@
  * GChemPaint library
  * bond.cc 
  *
- * Copyright (C) 2001-2010 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -31,22 +31,25 @@
 #include "theme.h"
 #include "view.h"
 #include "widgetdata.h"
-#include <canvas/gcp-canvas-group.h>
-#include <canvas/gcp-canvas-line.h>
-#include <canvas/gcp-canvas-bpath.h>
-#include <canvas/gcp-canvas-polygon.h>
+#include <gccv/canvas.h>
+#include <gccv/group.h>
+#include <gccv/hash.h>
+#include <gccv/line.h>
+#include <gccv/squiggle.h>
+#include <gccv/wedge.h>
 #include <gcu/cycle.h>
 #include <gcu/objprops.h>
 #include <glib/gi18n-lib.h>
 #include <cmath>
 #include <cstring>
 
+using namespace gccv;
 using namespace gcu;
 using namespace std;
 
 namespace gcp {
 
-Bond::Bond (): gcu::Bond ()
+Bond::Bond (): gcu::Bond (), ItemClient ()
 {
 	m_CoordsCalc = false;
 	m_type = NormalBondType;
@@ -54,7 +57,7 @@ Bond::Bond (): gcu::Bond ()
 }
 
 Bond::Bond (Atom* first, Atom* last, unsigned char order):
-		gcu::Bond (first, last, order)
+		gcu::Bond (first, last, order), ItemClient ()
 {
 	m_CoordsCalc = false;
 	m_type = NormalBondType;
@@ -110,13 +113,13 @@ bool Bond::GetLine2DCoords (unsigned Num, double* x1, double* y1, double* x2, do
 		if (ax > 0) {
 			horizontal = anga >= angb;
 			if (horizontal) {
-				ax = (ax + 2.) / Theme->GetZoomFactor ();
+				ax = (ax + 1.) / Theme->GetZoomFactor ();
 				if (dx > 0)
 					ax = - ax;
 				*x1 -= ax;
 				*y1 -= ax * dy / dx;
 			} else {
-				ay = (ay + 2.) / Theme->GetZoomFactor ();
+				ay = (ay + 1.) / Theme->GetZoomFactor ();
 				if (dy > 0)
 					ay = - ay;
 				*y1 -= ay;
@@ -207,13 +210,16 @@ general:
 				Bond *bond = reinterpret_cast <Bond*> (m_Begin->GetFirstBond (it));
 				if (bond == this)
 					bond = reinterpret_cast <Bond*> (m_Begin->GetNextBond (it));
-				double a0 = atan2 (*y1 - *y2, *x2 - *x1), a1 = bond->GetAngle2DRad (reinterpret_cast <Atom*> (m_Begin)), a;
+				double a0 = atan2 (*y1 - *y2, *x2 - *x1), a1 = bond->GetAngle2DRad (reinterpret_cast <Atom*> (m_Begin)), a2, a;
 				if (fabs (fabs (a0 - a1) - M_PI) > 0.01) {
 					double sign = sin (a0 - a1) > 0.0 ? 1.0 : -1.0;
 					double tanb = ((m_Begin->GetZ () == 6)? fabs (tan ((M_PI - a0 + a1) / 2)): 0.), cosa = cos (a0), sina = sin (a0);
 					m_coords[4] = *x1 + BondDist * cosa * tanb - dy * sign;
 					m_coords[5] = *y1 + dx * sign - BondDist * sina * tanb;
 					tanb = 0.;
+					a2 = M_PI + a0;
+					if (a2 > 2 * M_PI)
+						a2 -= 2 * M_PI;
 					bond = reinterpret_cast <Bond*> (m_End->GetFirstBond (it));
 					if (m_End->GetZ () == 6)
 						while (bond) {
@@ -225,8 +231,8 @@ general:
 							}
 							bond = reinterpret_cast <Bond*> (m_End->GetNextBond (it));
 						}
-					m_coords[6] = *x2 + (BondDist * cosa * tanb - dy) * sign;
-					m_coords[7] = *y2 +(dx - BondDist * sina * tanb) * sign;
+					m_coords[6] = *x2 + BondDist * cosa * tanb - dy * sign;
+					m_coords[7] = *y2 + dx * sign - BondDist * sina * tanb;
 					goto done;
 				}
 			} else if (n1 > 1 && n2 > 0) {
@@ -253,8 +259,8 @@ general:
 								}
 								bond = reinterpret_cast <Bond*> (m_Begin->GetNextBond (it));
 							}
-						m_coords[4] = *x1 - (BondDist * cosa * tanb + dy) * sign;
-						m_coords[5] = *y1 + (dx + BondDist * sina * tanb) * sign;
+						m_coords[4] = *x1 + BondDist * cosa * tanb - dy * sign;
+						m_coords[5] = *y1 + dx * sign - BondDist * sina * tanb;
 						tanb = (m_End->GetZ () == 6)? fabs (tan ((a1 - a0) / 2)): 0.;
 						m_coords[6] = *x2 - BondDist * cosa * tanb - dy * sign;
 						m_coords[7] = *y2 + dx * sign + BondDist * sina * tanb;
@@ -494,6 +500,11 @@ void Bond::RemoveAllCycles ()
 
 void Bond::Move (double x, double y, double z)
 {
+	if (m_Item) {
+		Document *doc = static_cast <Document*> (GetDocument ());
+		Theme *theme = doc->GetTheme ();
+		m_Item->Move (x * theme->GetZoomFactor (), y * theme->GetZoomFactor ());
+	}
 	m_CoordsCalc = false;
 }
 
@@ -510,15 +521,11 @@ void Bond::Revert ()
 	m_End = pAtom;
 }
 
-void Bond::SetSelected (GtkWidget* w, int state)
+void Bond::SetSelected (int state)
 {
 	if (!m_order)
 		return;
-	WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (w), "data");
-	GnomeCanvasGroup* group = pData->Items[this];
-	if (!group)
-		return; // this should not occur, but it might crash if it does occur after loading a bad file.
-	gchar const *color;
+	GOColor color;
 	switch (state) {	
 	case SelStateUnselected:
 		color = Color;
@@ -536,22 +543,144 @@ void Bond::SetSelected (GtkWidget* w, int state)
 		color = Color;
 		break;
 	}
-	void *obj = g_object_get_data (G_OBJECT (group), "path");
 	switch (GetType ())
 	{
-		case NormalBondType:
-		case UndeterminedBondType:
-			g_object_set (obj, "outline_color", color, NULL);
-			break;
-		case ForeBondType:
-		case UpBondType:
-		case DownBondType:
-			g_object_set (obj, "fill_color", color, NULL);
-			break;
+	case NormalBondType: {
+		gccv::Group *group = static_cast <gccv::Group *> (m_Item);
+		std::list<Item *>::iterator it;
+		gccv::Line *line = dynamic_cast <gccv::Line *> (group->GetFirstChild (it));
+		while (line) {
+			line->SetLineColor (color);
+			line = dynamic_cast <gccv::Line *> (group->GetNextChild (it));
+		}
+		break;
+	}
+	case UndeterminedBondType:
+	case ForeBondType: {
+		gccv::LineItem *item = static_cast <gccv::LineItem *> (m_Item);
+		item->SetLineColor (color);
+		break;
+	}
+	case UpBondType:{
+	case DownBondType:
+		gccv::Wedge *item = static_cast <gccv::Wedge *> (m_Item);
+		item->SetFillColor (color);
+		break;
+	}
 	}
 }
 
-void Bond::Add (GtkWidget* w) const
+void Bond::AddItem ()
+{
+	if (m_Item)
+		return;
+	double x1, y1, x2, y2;
+	Document *doc = static_cast <Document*> (GetDocument ());
+	View *view = doc->GetView ();
+	Theme *theme = doc->GetTheme ();
+	bool crossing = false;
+	if (m_Crossing.size () > 0) {
+		map<Bond*, BondCrossing>::const_iterator i, iend = m_Crossing.end ();
+		for (i = m_Crossing.begin (); i != iend; i++)
+			if ((crossing |= (*i).second.is_before))
+				break;
+	}
+	switch(GetType ()) {
+	case NormalBondType: {
+		gccv::Group *group = new gccv::Group (view->GetCanvas ()->GetRoot (), this);
+		m_Item = group;
+		int i = 1;
+		gccv::Line *line;
+		while (GetLine2DCoords (i++, &x1, &y1, &x2, &y2)) {
+			x1 *= theme->GetZoomFactor ();
+			y1 *= theme->GetZoomFactor ();
+			x2 *= theme->GetZoomFactor ();
+			y2 *= theme->GetZoomFactor ();
+			line = new gccv::Line (group, x1, y1, x2, y2, this);
+			line->SetLineWidth (theme->GetBondWidth ());
+			line->SetLineColor ((view->GetData ()->IsSelected (this))? SelectColor: Color);
+			if (crossing) {
+				// assuming that the crossing does not occur in the fisrt or last 10%
+				double dx = (x2 - x1) / 10., dy = (y2 - y1) / 10.;
+				line = new gccv::Line (group, x1 + dx, y1 + dy, x2 - dx, y2 -dy, this);
+				line->SetLineWidth (theme->GetBondWidth () * 3);
+				line->SetLineColor (0);
+				line->SetOperator (CAIRO_OPERATOR_SOURCE);
+				/* FIXME: unsecure if several bonds are crossing the same bond, use masking instead */
+				view->GetCanvas ()->GetRoot ()->MoveToBack (group);
+			}
+		}
+		break;
+	}
+	case UndeterminedBondType: {
+		GetLine2DCoords (1, &x1, &y1, &x2, &y2);
+		gccv::Squiggle *squiggle = new gccv::Squiggle (view->GetCanvas ()->GetRoot (),
+							x1 * theme->GetZoomFactor (),
+							y1 * theme->GetZoomFactor (),
+							x2 * theme->GetZoomFactor (),
+							y2 * theme->GetZoomFactor (),
+							this);
+		squiggle->SetLineWidth (theme->GetBondWidth ());
+		squiggle->SetWidth (theme->GetStereoBondWidth () - theme->GetBondWidth () / 2.);
+		squiggle->SetStep (theme->GetHashDist ());
+		squiggle->SetLineColor ((view->GetData ()->IsSelected (this))? SelectColor: Color);
+		m_Item = squiggle;
+		break;
+	}
+	case ForeBondType: {
+		GetLine2DCoords (1, &x1, &y1, &x2, &y2);
+		gccv::Line *line = new gccv::Line (view->GetCanvas ()->GetRoot (),
+							x1 * theme->GetZoomFactor (),
+							y1 * theme->GetZoomFactor (),
+							x2 * theme->GetZoomFactor (),
+							y2 * theme->GetZoomFactor (),
+							this);
+		line->SetLineWidth (theme->GetStereoBondWidth ());
+		line->SetLineColor ((view->GetData ()->IsSelected (this))? SelectColor: Color);
+		m_Item = line;
+		break;
+	}
+	case UpBondType: {
+		GetLine2DCoords (1, &x1, &y1, &x2, &y2);
+		gccv::Wedge *wedge = new gccv::Wedge (view->GetCanvas ()->GetRoot (),
+							x1 * theme->GetZoomFactor (),
+							y1 * theme->GetZoomFactor (),
+							x2 * theme->GetZoomFactor (),
+							y2 * theme->GetZoomFactor (),
+							theme->GetStereoBondWidth (),
+							this);
+		wedge->SetFillColor ((view->GetData ()->IsSelected (this))? SelectColor: Color);
+		m_Item = wedge;
+		break;
+	}
+	case DownBondType: {
+		GetLine2DCoords (1, &x1, &y1, &x2, &y2);
+		gccv::Hash *hash = new gccv::Hash (view->GetCanvas ()->GetRoot (),
+							x2 * theme->GetZoomFactor (),
+							y2 * theme->GetZoomFactor (),
+							x1 * theme->GetZoomFactor (),
+							y1 * theme->GetZoomFactor (),
+							theme->GetStereoBondWidth (),
+							this);
+		hash->SetFillColor ((view->GetData ()->IsSelected (this))? SelectColor: Color);
+		hash->SetLineWidth (theme->GetHashWidth ());
+		hash->SetLineDist (theme->GetHashDist ());
+		m_Item = hash;
+		break;
+	}
+	}
+}
+
+void Bond::UpdateItem ()
+{
+	if (m_Item) {
+		delete m_Item;
+		m_Item = NULL;
+	}
+	AddItem ();
+}
+
+/*void Bond::Add (GtkWidget* w) const
 {
 	if (!w)
 		return;
@@ -600,9 +729,9 @@ void Bond::Add (GtkWidget* w) const
 					NULL);
 			g_object_set_data (G_OBJECT (group), "back", item);
 			g_object_set_data (G_OBJECT (item), "object", (void *) this);
-			g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), w);
+			g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), w);*/
 			/* now bring to front, FIXME: not secure ! */
-			gnome_canvas_item_lower_to_bottom (item);
+/*			gnome_canvas_item_lower_to_bottom (item);
 			gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (group));
 			gcu::Atom *pAtom = GetAtom (0);
 			if (pAtom->GetZ () != 6 || static_cast <gcp::Atom*> (pAtom)->GetShowSymbol ())
@@ -666,9 +795,9 @@ void Bond::Add (GtkWidget* w) const
 		gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (pData->Items[electron]));
 		electron = pAtom1->GetNextChild (i);
 	}
-}
+}*/
 
-GnomeCanvasPathDef* Bond::BuildPathDef (WidgetData* pData)
+/*GnomeCanvasPathDef* Bond::BuildPathDef (WidgetData* pData)
 {
 	double x1, y1, x2, y2, x, y, dx, dy, dx1, dy1;
 	int i, n;
@@ -831,9 +960,9 @@ GnomeCanvasPathDef *Bond::BuildCrossingPathDef (WidgetData* pData)
 		break;
 	}
 	return path;
-}
+}*/
 
-void Bond::Update(GtkWidget* w) const
+/*void Bond::Update(GtkWidget* w) const
 {
 	if (!w || !m_order)
 		return;
@@ -879,9 +1008,9 @@ void Bond::Update(GtkWidget* w) const
 						NULL);
 				g_object_set_data (G_OBJECT (group), "back", item);
 				g_object_set_data (G_OBJECT (item), "object", (void *) this);
-				g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), w);
+				g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), w);*/
 				/* now bring to front, FIXME: not secure ! */
-				gnome_canvas_item_lower_to_bottom (item);
+/*				gnome_canvas_item_lower_to_bottom (item);
 				gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (group));
 				gcu::Atom *pAtom = GetAtom (0);
 				if (pAtom->GetZ () != 6 || static_cast <gcp::Atom*> (pAtom)->GetShowSymbol ())
@@ -897,13 +1026,11 @@ void Bond::Update(GtkWidget* w) const
 	}
 	path = const_cast <Bond *> (this)->BuildPathDef (pData);
 	obj = g_object_get_data(G_OBJECT(group), "path");
-	if (obj) {
-		g_object_set (obj, "bpath", path, NULL);
-		if (m_type == NormalBondType || m_type == UndeterminedBondType)
-			g_object_set (obj, "width_units", pTheme->GetBondWidth (), NULL);
-	}
+	g_object_set (obj, "bpath", path, NULL);
+	if (m_type == NormalBondType || m_type == UndeterminedBondType)
+		g_object_set (obj, "width_units", pTheme->GetBondWidth (), NULL);
 	gnome_canvas_path_def_unref (path);
-}
+}*/
 	
 double Bond::GetYAlign ()
 {
