@@ -24,34 +24,35 @@
 
 #include "config.h"
 #include "application.h"
-#include "view.h"
-#include "document.h"
-#include "settings.h"
+#include "atom.h"
+#include "bond.h"
 #include "docprop.h"
+#include "document.h"
 #include "fragment.h"
-#include "reaction.h"
 #include "mesomery.h"
 #include "molecule.h"
+#include "reaction.h"
 #include "residue.h"
+#include "settings.h"
 #include "text.h"
 #include "theme.h"
 #include "tool.h"
+#include "view.h"
+#include "widgetdata.h"
 #include "window.h"
-#include <gcu/objprops.h>
 #include <gcu/loader.h>
-#ifndef GOFFICE_HAS_GLOBAL_HEADER
-#   include <goffice/math/go-rangefunc.h>
-#   include <gtk/gtk.h>
-#   include <gio/gio.h>
-#endif
-#include <unistd.h>
-#include <libgen.h>
+#include <gcu/objprops.h>
+#include <goffice/math/go-rangefunc.h>
+#include <gtk/gtk.h>
+#include <gio/gio.h>
 #include <glib/gi18n-lib.h>
 #include <openbabel/mol.h>
 #include <openbabel/obconversion.h>
 #include <clocale>
 #include <stack>
 #include <cstring>
+#include <libgen.h>
+#include <unistd.h>
 
 using namespace OpenBabel;
 using namespace std;
@@ -161,10 +162,8 @@ const gchar* Document::GetTitle () const
 {
 	if (m_title)
 		return m_title;
-	else if (m_label)
-		return m_label;
 	else
-		return m_Window->GetDefaultTitle ();
+		return GetLabel ();
 }
 
 const gchar* Document::GetLabel () const
@@ -361,7 +360,7 @@ void Document::ExportOB () const
 				Mol.SetTitle ((char*) GetTitle ());
 				Mol.SetDimension (2);
 				for (i = AtomTable.begin (); i != AtomTable.end (); i++) {
-					pgAtom = (Atom*) Ob->GetDescendant ((*i).first.c_str ());
+					pgAtom = (Atom*) Ob->GetDescendant ((*i).first.data ());
 					obAtom.SetIdx ((*i).second);
 					obAtom.SetAtomicNum (pgAtom->GetZ ());
 					pgAtom->GetCoords (&x, &y, &z);
@@ -430,9 +429,9 @@ void Document::DoPrint (GtkPrintOperation *print, GtkPrintContext *context) cons
 	// adjust position and size
 	GtkWidget* widget = m_pView->GetWidget ();
 	WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (widget), "data");
-	ArtDRect rect;
+	gccv::Rect rect;
 	pData->GetObjectBounds (this, &rect);
-	double scale = .75;
+	double scale = .75; // FIXME: should we use the real screen resolution?
 	x = rect.x0;
 	y = rect.y0;
 	w = rect.x1 - rect.x0;
@@ -483,7 +482,10 @@ void Document::AddAtom (Atom* pAtom)
 		pAtom->SetId (id);
 		Id = id;
 	}
-	m_pView->AddObject (pAtom);
+	if (!pAtom->GetParent ())
+		AddChild (pAtom);
+	if (m_pView->GetCanvas ())
+		m_pView->AddObject (pAtom);
 	if (m_bIsLoading)
 		return;
 	Molecule* mol = new Molecule ();
@@ -544,9 +546,11 @@ void Document::AddBond (Bond* pBond)
 	}
 	AddChild (pBond);
 	Atom *pAtom0 = (Atom*) pBond->GetAtom (0), *pAtom1 = (Atom*) pBond->GetAtom (1);
-	m_pView->Update (pAtom0);
-	m_pView->Update (pAtom1);
-	m_pView->AddObject (pBond);
+	if (m_pView->GetCanvas ()) {
+		pAtom0->UpdateItem ();
+		pAtom1->UpdateItem ();
+		pBond->AddItem ();
+	}
 	if (m_bIsLoading)
 		return;
 	//search molecules
@@ -1290,7 +1294,7 @@ double Document::GetYAlign ()
 		return Child->GetYAlign ();
 	} else {
 		WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (GetWidget ()), "data");
-		ArtDRect rect;
+		gccv::Rect rect;
 		pData->GetObjectBounds (this, &rect);
 		return (rect.y1 - rect.y0) / 2.;
 	}
@@ -1298,12 +1302,12 @@ double Document::GetYAlign ()
 
 void Document::SetTheme (Theme *theme)
 {
-	if (m_Theme)
-		m_Theme->RemoveClient (this);
 	if (!theme) {
 		m_Theme = NULL;
 		return;
 	}
+	if (m_Theme)
+		m_Theme->RemoveClient (this);
 	m_Theme = theme;
 	m_Theme->AddClient (this);
 	m_BondLength = theme->GetBondLength ();
@@ -1404,7 +1408,7 @@ double Document::GetMedianBondLength ()
 		}
 	}
 	if (nb > 0)
-		go_range_median_inter_nonconst (&lengths[0], nb, &result);
+		go_range_median_inter_nonconst (lengths.data (), nb, &result);
 	return result;
 }
 
