@@ -35,8 +35,7 @@
 #ifndef HAVE_GO_CONF_SYNC
 #include <gconf/gconf-client.h>
 #endif
-#include <gtk/gtklabel.h>
-#include <gtk/gtkspinbutton.h>
+#include <gtk/gtk.h>
 #include <glib/gi18n-lib.h>
 #include <sys/stat.h>
 #include <cmath>
@@ -80,16 +79,12 @@ Application::Application (string name, string datadir, char const *help_name, ch
 		HelpFilename = string ("file://") + datadir + string ("/gnome/help/") + HelpName + string ("-"API_VERSION"/C/") + HelpName + "-"API_VERSION".xml";
 	}
 	HelpBrowser = "yelp"; // there is no more key for that
-	char *dir = g_get_current_dir ();
-	char *uri = g_filename_to_uri (dir, NULL, NULL);
-	g_free (dir);
-	CurDir = uri;
-	g_free (uri);
+	CurDir = g_get_current_dir ();
 	if (first_call) { // needed to create several applications in the same program instance
 		g_set_application_name (name.c_str ());
 		first_call = false;
 	}
-	gtk_window_set_default_icon_name (icon_name? icon_name: (help_name? help_name: Name.c_str ()));
+	IconName = icon_name? icon_name: (help_name? help_name: Name.c_str ());
 	GdkScreen *screen = gdk_screen_get_default ();
 	m_ScreenResolution = (unsigned) rint (gdk_screen_get_width (screen) * 25.4 / gdk_screen_get_width_mm (screen));
 	m_ImageResolution = m_ScreenResolution;
@@ -137,7 +132,7 @@ void Application::OnHelp (string tag)
 		path += string("#") + HelpName + string ("-") + tag;
 	argv[1] = (char*) path.c_str ();
 	g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
-		NULL, NULL, NULL, NULL);
+				   NULL, NULL, NULL, NULL);
 }
 
 bool Application::HasHelp ()
@@ -190,6 +185,11 @@ static void on_height_changed (GtkSpinButton *btn, Application *app)
 	app->SetImageHeight (gtk_spin_button_get_value_as_int (btn));
 }
 
+static void on_transparency_changed (GtkToggleButton *btn, Application *app)
+{
+	app->SetTransparentBackground (gtk_toggle_button_get_active (btn));
+}
+
 GtkWidget *Application::GetImageResolutionWidget ()
 {
 	GladeXML *xml = glade_xml_new (GLADEDIR"/image-resolution.glade", "res-table", NULL);
@@ -200,6 +200,9 @@ GtkWidget *Application::GetImageResolutionWidget ()
 	w = glade_xml_get_widget (xml, "res-btn");
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (w), m_ImageResolution);
 	g_signal_connect (G_OBJECT (w), "value-changed", G_CALLBACK (on_res_changed), this);
+	w = glade_xml_get_widget (xml, "transparent-btn");
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), m_TransparentBackground);
+	g_signal_connect (G_OBJECT (w), "toggled", G_CALLBACK (on_transparency_changed), this);
 	w = glade_xml_get_widget (xml, "res-table");
 	g_object_unref (G_OBJECT (xml));
 	return w;
