@@ -30,11 +30,7 @@
 #include <set>
 #include <string>
 #ifdef HAVE_GO_CONF_SYNC
-#   ifdef GOFFICE_HAS_GLOBAL_HEADER
-#      include <goffice/goffice.h>
-#   else
-#      include <goffice/app/go-conf.h>
-#   endif
+#include <goffice/app/go-conf.h>
 #endif
 #include <gtk/gtkmain.h>
 #include <gtk/gtkwindow.h>
@@ -52,7 +48,7 @@ class Dialog;
 /*!\class Application gcu/application.h
 This class is a base class for applications. It provides some basic services.
 */
-class Application: virtual public DialogOwner
+class Application: public DialogOwner
 {
 friend class Document;
 friend class Dialog;
@@ -228,6 +224,8 @@ Method used to retrieve the base configuration node.
 	static GOConfNode *GetConfDir ();
 #endif
 
+	std::string const &GetIconName () {return IconName;}
+
 protected:
 
 /*!
@@ -247,6 +245,7 @@ private:
 	std::string HelpBrowser;
 	std::string HelpFilename;
 	std::string CurDir;
+	std::string IconName;
 #ifdef HAVE_GO_CONF_SYNC
 	static GOConfNode *m_ConfDir;
 #endif
@@ -310,6 +309,18 @@ an exported image size, but not both.
 @return the current image height used on export as a reference.
 */
 GCU_PROP (unsigned, ImageHeight)
+/*!\fn SetTransparentBackground(bool transpasrent)
+@param transparent whether an export image should have a transparent background.
+
+Sets the transparency of the exported image when possible.
+*/
+/*!\fn GetTransparentBackground()
+@return whether exported image have a transparent background.
+*/
+/*!\fn GetRefTransparentBackground()
+@return whether exported image have a transparent background as a reference.
+*/
+GCU_PROP (bool, TransparentBackground)
 /*!\fn GetRecentManager()
 @return the GtkRecentFileManager attached to the application.
 */
