@@ -23,11 +23,7 @@
 #include "config.h"
 #include "element.h"
 #include "xml-utils.h"
-#ifdef GOFFICE_HAS_GLOBAL_HEADER
-#   include <goffice/goffice.h>
-#else
-#   include <goffice/math/go-math.h>
-#endif
+#include <goffice/math/go-math.h>
 #include <libxml/parser.h>
 #include <libxml/parserInternals.h>
 #include <libxml/xmlmemory.h>
@@ -48,8 +44,8 @@ static set<string>units;
 
 static void ReadValue (char const *source, GcuValue &value)
 {
-	const char *buf, *dot;
-	value.value = strtod (source, const_cast <char**> (&buf));
+	char *buf, *dot;
+	value.value = strtod (source, &buf);
 	dot = strchr (source, '.');
 	value.prec = (dot)? buf - dot - 1: 0;
 	value.delta = (*buf == '(')? strtol (buf + 1, NULL, 10): 0;
@@ -57,8 +53,8 @@ static void ReadValue (char const *source, GcuValue &value)
 
 static void ReadDimensionalValue (char const *source, GcuDimensionalValue &value)
 {
-	const char *buf, *dot;
-	value.value = strtod (source, const_cast <char**> (&buf));
+	char *buf, *dot;
+	value.value = strtod (source, &buf);
 	dot = strchr (source, '.');
 	value.prec = (dot)? buf - dot - 1: 0;
 	value.delta = (*buf == '(')? strtol (buf + 1, NULL, 10): 0;
