@@ -34,8 +34,7 @@
 #include <goffice/math/go-rangefunc.h>
 #include <goffice/math/go-regression.h>
 #include <goffice/utils/go-color.h>
-#include <libgnomevfs/gnome-vfs-ops.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
+#include <gio/gio.h>
 #include <glib/gi18n-lib.h>
 #include <cstring>
 #include <sstream>
@@ -46,7 +45,11 @@ using namespace std;
 namespace gcu
 {
 
-SpectrumDocument::SpectrumDocument (): Document (NULL), Printable (), m_Empty (true)
+SpectrumDocument::SpectrumDocument ():
+	Document (NULL),
+	Printable (),
+	m_XAxisInvertBtn (NULL),
+	m_Empty (true)
 {
 	m_View = new SpectrumView (this);
 	x = y = NULL;
@@ -54,7 +57,7 @@ SpectrumDocument::SpectrumDocument (): Document (NULL), Printable (), m_Empty (t
 	npoints = 0;
 	maxx = maxy = minx = miny = go_nan;
 	firstx = lastx = deltax = firsty = go_nan;
-	freq = go_nan;
+	freq = offset = refpoint = go_nan;
 	gtk_page_setup_set_orientation (GetPageSetup (), GTK_PAGE_ORIENTATION_LANDSCAPE);
 	SetScaleType (GCU_PRINT_SCALE_AUTO);
 	SetHorizFit (true);
@@ -65,6 +68,7 @@ SpectrumDocument::SpectrumDocument (): Document (NULL), Printable (), m_Empty (t
 SpectrumDocument::SpectrumDocument (Application *App, SpectrumView *View):
 	Document (App),
 	Printable (),
+	m_XAxisInvertBtn (NULL),
 	m_Empty (true)
 {
 	m_View = (View)? View: new SpectrumView (this);
@@ -73,7 +77,7 @@ SpectrumDocument::SpectrumDocument (Application *App, SpectrumView *View):
 	npoints = 0;
 	maxx = maxy = minx = miny = go_nan;
 	firstx = lastx = deltax = firsty = go_nan;
-	freq = go_nan;
+	freq = offset = refpoint = go_nan;
 	gtk_page_setup_set_orientation (GetPageSetup (), GTK_PAGE_ORIENTATION_LANDSCAPE);
 	SetScaleType (GCU_PRINT_SCALE_AUTO);
 	SetHorizFit (true);
@@ -98,29 +102,50 @@ void SpectrumDocument::Load (char const *uri, char const *mime_type)
 	if (!mime_type || strcmp (mime_type, "chemical/x-jcamp-dx")) {
 		return;
 	}
-	GnomeVFSHandle *handle;
-	GnomeVFSFileInfo *info = gnome_vfs_file_info_new ();
-	GnomeVFSResult result = gnome_vfs_open (&handle, uri, GNOME_VFS_OPEN_READ);
-	if (result != GNOME_VFS_OK) {
-		gnome_vfs_file_info_unref (info);
+	GVfs *vfs = g_vfs_get_default ();
+	GFile *file = g_vfs_get_file_for_uri (vfs, uri);
+	GError *error = NULL;
+	GFileInfo *info = g_file_query_info (file,
+										 G_FILE_ATTRIBUTE_STANDARD_SIZE,
+										 G_FILE_QUERY_INFO_NONE,
+										 NULL, &error);
+	if (error) {
+		g_message ("GIO querry failed: %s", error->message);
+		g_error_free (error);
+		g_object_unref (file);
 		return;
 	}
-	gnome_vfs_get_file_info_from_handle (handle, info, (GnomeVFSFileInfoOptions) 0);
-	gchar *buf = new gchar[info->size + 1];
-	GnomeVFSFileSize n;
-	gnome_vfs_read (handle, buf, info->size, &n);
-	buf[info->size] = 0;
-	if (n == info->size) {
-		LoadJcampDx (buf);
-		if (m_App) {
-			char *dirname = g_path_get_dirname (uri);
-			m_App->SetCurDir (dirname);
-			g_free (dirname);
+	gsize size = g_file_info_get_size (info);
+	g_object_unref (info);
+	GInputStream *input = G_INPUT_STREAM (g_file_read (file, NULL, &error));
+	if (error) {
+		g_message ("GIO could not create the stream: %s", error->message);
+		g_error_free (error);
+		g_object_unref (file);
+		return;
+	}
+	gchar *buf = new gchar[size + 1];
+	gsize n = size;
+	while (n) {
+		n -= g_input_stream_read (input, buf, size, NULL, &error);
+		if (error) {
+			g_message ("GIO could not read the file: %s", error->message);
+			g_error_free (error);
+			delete [] buf;
+			g_object_unref (input);
+			g_object_unref (file);
+			return;
 		}
 	}
-	gnome_vfs_file_info_unref (info);
+	buf[size] = 0;
+	LoadJcampDx (buf);
+	if (m_App) {
+		char *dirname = g_path_get_dirname (uri);
+		m_App->SetCurDir (dirname);
+		g_free (dirname);
+	}
 	delete [] buf;
-	g_free (handle);
+	g_object_unref (file);
 	
 }
 
@@ -374,6 +399,9 @@ char const *Keys[] = {
 	"OBSERVE90",
 	"COUPLINGCONSTANTS",
 	"RELAXATIONTIMES",
+/* Brücker specific data */
+	"$OFFSET",
+	"$REFERENCEPOINT",
 	NULL
 };
 
@@ -502,6 +530,9 @@ enum {
 	JCAMP_OBSERVE_90,
 	JCAMP_COUPLING_CONSTANTS,
 	JCAMP_RELAXATION_TIMES,
+	BRUCKER_OFFSET,
+	VARIAN_OFFSET,
+	JCAMP_MAX_VALID
 };
 
 #define KEY_LENGTH 80
@@ -518,15 +549,19 @@ static int ReadField (char const *s, char *key, char *buf)
 		// This is a comment, just skip
 		return JCAMP_INVALID;
 	}
-	if (!strncmp (data, "##$", 3)) {
+/*	if (!strncmp (data, "##$", 3)) {
 		// This is a vendor specific tag, just skip for now
 		return JCAMP_INVALID;
-	}
+	}*/
 	if (!strncmp (data, "##", 2)) {
 		// found a field
 		data += 2;
 		eq = strchr (data, '=');
 		i = 0;
+		if (*data == '$' && strncmp (data, "$$", 2)) {
+			key[i++] = '$';
+			data++;
+		}
 		while (data < eq) {
 			if (*data >= 'A' && *data <= 'Z')
 				key[i++] = *data;
@@ -555,16 +590,26 @@ static int ReadField (char const *s, char *key, char *buf)
 		
 		// Now, get the key value
 		i = JCAMP_TITLE;
-		while (i <= JCAMP_RELAXATION_TIMES && strcmp (key, Keys[i]))
+		while (i < JCAMP_MAX_VALID && strcmp (key, Keys[i]))
 			i++;
 		return i;
 	}
 	return JCAMP_UNKNOWN;
 }
 
-static void on_unit_changed (GtkComboBox *box, SpectrumDocument *doc)
+static void on_xunit_changed (GtkComboBox *box, SpectrumDocument *doc)
 {
-	doc->OnUnitChanged (gtk_combo_box_get_active (box));
+	doc->OnXUnitChanged (gtk_combo_box_get_active (box));
+}
+
+static void on_yunit_changed (GtkComboBox *box, SpectrumDocument *doc)
+{
+	doc->OnYUnitChanged (gtk_combo_box_get_active (box));
+}
+
+static void on_xaxis_invert (GtkToggleButton *btn, SpectrumDocument *doc)
+{
+	doc->OnXAxisInvert (gtk_toggle_button_get_active (btn));
 }
 
 static void on_show_integral (GtkButton *btn, SpectrumDocument *doc)
@@ -574,10 +619,13 @@ static void on_show_integral (GtkButton *btn, SpectrumDocument *doc)
 	doc->OnShowIntegral ();
 }
 
+/*
 static void on_transform_fid (GtkButton *btn, SpectrumDocument *doc)
 {
 	doc->OnTransformFID (btn);
 }
+*/
+
 #define JCAMP_PREC 1e-3 // fully arbitrary
 
 void SpectrumDocument::LoadJcampDx (char const *data)
@@ -598,6 +646,7 @@ void SpectrumDocument::LoadJcampDx (char const *data)
 	var.Values = NULL;
 	while (!s.eof ()) {
 		s.getline (line, 300);
+parse_line:
 		n = ReadField (line, key, buf);
 		switch (n) {
 		case JCAMP_TITLE:
@@ -655,22 +704,21 @@ void SpectrumDocument::LoadJcampDx (char const *data)
 						break;	// this should not occur, but a corrupted or bad file is always possible
 					s.getline (line, 300);
 					if (strstr (line, "##")) {
-						s.seekg (-strlen (line) -1, _S_cur);
 						if (read > npoints) {
 							g_warning (_("Found too many data!"));
 							// FIXME: throw an exception
 						} else
 							npoints = read;
-						break;
+						goto parse_line;
 					}
 					cur = line;
 					while (*cur) {
-						while (*cur && (*cur < '0' || *cur > '9'))
+						while (*cur && (*cur < '0' || *cur > '9') && *cur != '-' && *cur !='+')
 							cur++;
 						if (*cur == 0)
 							break;
 						x[read] = strtod (cur, &cur);
-						while (*cur && (*cur < '0' || *cur > '9'))
+						while (*cur && (*cur < '0' || *cur > '9') && *cur != '-' && *cur !='+')
 							cur++;
 						y[read] = strtod (cur, &cur);
 						read++;
@@ -1163,6 +1211,13 @@ void SpectrumDocument::LoadJcampDx (char const *data)
 		case JCAMP_OBSERVE_90:
 		case JCAMP_COUPLING_CONSTANTS:
 		case JCAMP_RELAXATION_TIMES:
+			break;
+		case BRUCKER_OFFSET:
+			offset = strtod (buf, NULL);
+			break;
+		case VARIAN_OFFSET:
+			refpoint = strtod (buf, NULL);
+			break;
 		default:
 			break;
 		}
@@ -1170,11 +1225,51 @@ void SpectrumDocument::LoadJcampDx (char const *data)
 
 out:
 	bool hide_y_axis = false;
+	SpectrumUnitType unit = GCU_SPECTRUM_UNIT_MAX;
 	// doon't do anything for unsupported spectra
 	switch (m_SpectrumType) {
 	case GCU_SPECTRUM_NMR: {
 		if (x == NULL && X > 0 && variables[X].Values == NULL)
 			return;
+		// fix origin
+		if (go_finite (offset)) {
+			unsigned i;
+			if (x) {
+				double d = offset * freq - maxx;
+				maxx += d;
+				minx += d;
+				firstx += d;
+				lastx += d;
+				for (i = 0; i < npoints; i++)
+					x[i] += d;
+			} else {
+				double d = offset * freq - variables[X].Max;
+				maxx = variables[X].Max += d;
+				minx = variables[X].Min += d;
+				variables[X].First += d;
+				variables[X].Last += d;
+				for (i = 0; i < npoints; i++)
+					variables[X].Values[i] += d;
+			}
+		} else if (go_finite (refpoint)) {
+			unsigned i;
+			double d = -refpoint;
+			if (x) {
+				maxx += d;
+				minx += d;
+				firstx += d;
+				lastx += d;
+				for (i = 0; i < npoints; i++)
+					x[i] += d;
+			} else {
+				maxx = variables[X].Max += d;
+				minx = variables[X].Min += d;
+				variables[X].First += d;
+				variables[X].Last += d;
+				for (i = 0; i < npoints; i++)
+					variables[X].Values[i] += d;
+			}
+		}
 		// add some widgets to the option box
 		GtkWidget *box = gtk_hbox_new (false, 5), *w;
 		if (go_finite (freq)) {
@@ -1185,7 +1280,7 @@ out:
 			gtk_combo_box_append_text (GTK_COMBO_BOX (w), _("Frequency (Hz)"));
 			SpectrumUnitType unit = (X >= 0)? variables[X].Unit: m_XUnit;
 			gtk_combo_box_set_active (GTK_COMBO_BOX (w), ((unit == GCU_SPECTRUM_UNIT_PPM)? 0: 1));
-			g_signal_connect (w, "changed", G_CALLBACK (on_unit_changed), this);
+			g_signal_connect (w, "changed", G_CALLBACK (on_xunit_changed), this);
 			gtk_box_pack_start (GTK_BOX (box), w, false, false, 0);
 		}
 		w = gtk_button_new_with_label (_("Show integral"));
@@ -1199,7 +1294,8 @@ out:
 	case GCU_SPECTRUM_NMR_FID: {
 		if (x == NULL && X > 0 && variables[X].Values == NULL)
 			return;
-		if (R >= 0 && I >= 0) {
+// Deactivate Fourier transfor for now
+/*		if (R >= 0 && I >= 0) {
 			GtkWidget *box = gtk_hbox_new (false, 5);
 			GtkWidget *w = gtk_button_new_with_label (_("Transform to spectrum"));
 			g_signal_connect (w, "clicked", G_CALLBACK (on_transform_fid), this);
@@ -1207,19 +1303,53 @@ out:
 			gtk_widget_show_all (box);
 			gtk_box_pack_start (GTK_BOX (m_View->GetOptionBox ()), box, false, false, 0);
 		}
-		hide_y_axis = true;
+		hide_y_axis = true;*/
 		break;
 	}
 	case GCU_SPECTRUM_INFRARED:
 	case GCU_SPECTRUM_RAMAN:
+		unit = GCU_SPECTRUM_UNIT_MICROMETERS;
 //	case GCU_SPECTRUM_INFRARED_PEAK_TABLE:
 //	case GCU_SPECTRUM_INFRARED_INTERFEROGRAM:
 //	case GCU_SPECTRUM_INFRARED_TRANSFORMED:
 	case GCU_SPECTRUM_UV_VISIBLE:
 		if (x == NULL && X > 0 && variables[X].Values == NULL)
 			return;
-		else
-			break;
+		else {
+			if (unit == GCU_SPECTRUM_UNIT_MAX)
+				unit = GCU_SPECTRUM_UNIT_NANOMETERS;
+			// add some widgets to the option box
+			GtkWidget *box = gtk_hbox_new (false, 5), *w;
+			w = gtk_label_new (_("X unit:"));
+			gtk_box_pack_start (GTK_BOX (box), w, false, false, 0);
+			w = gtk_combo_box_new_text ();
+			if  (unit == GCU_SPECTRUM_UNIT_NANOMETERS)
+				gtk_combo_box_append_text (GTK_COMBO_BOX (w), _("Wave length (nm)"));
+			else
+				gtk_combo_box_append_text (GTK_COMBO_BOX (w), _("Wave length (µm)"));
+			gtk_combo_box_append_text (GTK_COMBO_BOX (w), _("Wave number (1/cm)"));
+			SpectrumUnitType unit = (X >= 0)? variables[X].Unit: m_XUnit;
+			gtk_combo_box_set_active (GTK_COMBO_BOX (w), ((unit == GCU_SPECTRUM_UNIT_CM_1)? 1: 0));
+			g_signal_connect (w, "changed", G_CALLBACK (on_xunit_changed), this);
+			gtk_box_pack_start (GTK_BOX (box), w, false, false, 0);
+			m_XAxisInvertBtn = gtk_check_button_new_with_label (_("Invert X Axis"));
+			m_XAxisInvertSgn = g_signal_connect (m_XAxisInvertBtn, "toggled", G_CALLBACK (on_xaxis_invert), this);
+			gtk_box_pack_start (GTK_BOX (box), m_XAxisInvertBtn, false, false, 0);
+			w = gtk_vseparator_new ();
+			gtk_box_pack_start (GTK_BOX (box), w, false, false, 12);
+			w = gtk_label_new (_("Y unit:"));
+			gtk_box_pack_start (GTK_BOX (box), w, false, false, 0);
+			w = gtk_combo_box_new_text ();
+			gtk_combo_box_append_text (GTK_COMBO_BOX (w), _("Absorbance"));
+			gtk_combo_box_append_text (GTK_COMBO_BOX (w), _("Transmittance"));
+			unit = (Y >= 0)? variables[Y].Unit: m_YUnit;
+			gtk_combo_box_set_active (GTK_COMBO_BOX (w), ((unit == GCU_SPECTRUM_UNIT_ABSORBANCE)? 0: 1));
+			g_signal_connect (w, "changed", G_CALLBACK (on_yunit_changed), this);
+			gtk_box_pack_start (GTK_BOX (box), w, false, false, 0);
+			gtk_widget_show_all (box);
+			gtk_box_pack_start (GTK_BOX (m_View->GetOptionBox ()), box, false, false, 0);
+		}
+		break;
 //	case GCU_SPECTRUM_NMR_PEAK_TABLE:
 //	case GCU_SPECTRUM_NMR_PEAK_ASSIGNMENTS:
 	case GCU_SPECTRUM_MASS:
@@ -1260,7 +1390,13 @@ out:
 	default:
 		break;
 	}
+	if (m_XAxisInvertBtn) {
+		g_signal_handler_block (m_XAxisInvertBtn, m_XAxisInvertSgn);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (m_XAxisInvertBtn), invert_axis);
+		g_signal_handler_unblock (m_XAxisInvertBtn, m_XAxisInvertSgn);
+	}
 	m_View->SetAxisBounds (GOG_AXIS_X, minx, maxx, invert_axis);
+	m_View->SetAxisBounds (GOG_AXIS_Y, miny, maxy, invert_axis);
 	if (hide_y_axis)
 		m_View->ShowAxis (GOG_AXIS_Y, false);
 	/* Add axes labels */
@@ -1545,7 +1681,7 @@ void SpectrumDocument::ReadDataTable (istream &s, double *x, double *y)
 	}
 }
 
-void SpectrumDocument::OnUnitChanged (int i)
+void SpectrumDocument::OnXUnitChanged (int i)
 {
 	SpectrumUnitType unit = GCU_SPECTRUM_UNIT_MAX;
 	bool invert_axis = false;
@@ -1554,6 +1690,21 @@ void SpectrumDocument::OnUnitChanged (int i)
 		unit = (i == 0)? GCU_SPECTRUM_UNIT_PPM: GCU_SPECTRUM_UNIT_HZ;
 		invert_axis = true;
 		break;
+	case GCU_SPECTRUM_INFRARED:
+	case GCU_SPECTRUM_RAMAN:
+		if (i == 1) {
+			unit = GCU_SPECTRUM_UNIT_CM_1;
+			invert_axis = true;
+		} else
+			unit = GCU_SPECTRUM_UNIT_MICROMETERS;
+		break;
+	case GCU_SPECTRUM_UV_VISIBLE:
+		if (i == 1) {
+			unit = GCU_SPECTRUM_UNIT_CM_1;
+			invert_axis = true;
+		} else
+			unit = GCU_SPECTRUM_UNIT_NANOMETERS;
+		break;
 	default:
 		break;
 	}
@@ -1561,14 +1712,21 @@ void SpectrumDocument::OnUnitChanged (int i)
 		return;
 	GOData *godata;
 	GogSeries *series = m_View->GetSeries ();
-	if (X < 0 && m_XUnit == unit) {
+	if (x && m_XUnit == unit) {
+		X = -1;
 		godata = go_data_vector_val_new (x, npoints, NULL);
 		gog_series_set_dim (series, 0, godata, NULL);
 		m_View->SetAxisBounds (GOG_AXIS_X, minx, maxx, invert_axis);
 		m_View->SetAxisLabel (GOG_AXIS_X, _(UnitNames[m_XUnit]));
+		if (m_XAxisInvertBtn) {
+			g_signal_handler_block (m_XAxisInvertBtn, m_XAxisInvertSgn);
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (m_XAxisInvertBtn), invert_axis);
+			g_signal_handler_unblock (m_XAxisInvertBtn, m_XAxisInvertSgn);
+		}
 	} else {
 		unsigned i, j;
-		double f;
+		double (*conv) (double, double, double);
+		double f, o;
 		for (i = 0; i < variables.size (); i++)
 			if (variables[i].Symbol == 'X' && variables[i].Unit == unit)
 				break;
@@ -1576,44 +1734,59 @@ void SpectrumDocument::OnUnitChanged (int i)
 			// Add new data vector
 			JdxVar v;
 			if (X >=0) {
-				f = GetConversionFactor (variables[X].Unit, unit);
+				conv = GetConversionFunction (variables[X].Unit, unit, f, o);
+				if (!conv)
+					return;
 				v.Name = _(UnitNames[variables[X].Unit]);
 				v.Symbol = variables[X].Symbol;
 				v.Type = variables[X].Type;
 				v.Unit = unit;
 				v.Format = variables[X].Format;
 				v.NbValues = variables[X].NbValues;
-				v.First = variables[X].First * f;
-				v.Last = variables[X].Last * f; 
-				v.Min = variables[X].Min * f;
-				v.Max = variables[X].Max * f;
-				v.Factor = 1.;
-				v.Values = new double[variables[X].NbValues];
-				for (j = 0; j < variables[X].NbValues; j++)
-					v.Values[j] = variables[X].Values[j] * f;
+					v.First = conv (variables[X].First, f, o);
+					v.Last = conv (variables[X].Last, f, o); 
+					v.Min = conv (variables[X].Min, f, o);
+					v.Max = conv (variables[X].Max, f, o);
+					v.Factor = 1.;
+					v.Values = new double[variables[X].NbValues];
+					for (j = 0; j < variables[X].NbValues; j++)
+						v.Values[j] = conv (variables[X].Values[j], f, o);
 			} else {
-				f = GetConversionFactor (m_XUnit, unit);
+				conv = GetConversionFunction (m_XUnit, unit, f, o);
+				if (!conv)
+					return;
 				v.Name = _(UnitNames[unit]);
 				v.Symbol = 'X';
 				v.Type = GCU_SPECTRUM_TYPE_DEPENDENT;
 				v.Unit = unit;
 				v.Format = GCU_SPECTRUM_FORMAT_MAX;
 				v.NbValues = npoints;
-				v.First = firstx * f;
-				v.Last = lastx * f; 
-				v.Min = minx * f;
-				v.Max = maxx * f;
-				v.Factor = 1.;
-				v.Values = new double[npoints];
-				for (j = 0; j < npoints; j++)
-					v.Values[j] = x[j] * f;
+					v.First = conv (firstx, f, o);
+					v.Last = conv (lastx, f, o); 
+					v.Min = conv (minx, f, o);
+					v.Max = conv (maxx, f, o);
+					v.Factor = 1.;
+					v.Values = new double[npoints];
+					for (j = 0; j < npoints; j++)
+						v.Values[j] = conv (x[j], f, o);
+			}
+			if (v.Min > v.Max) {
+				f = v.Min;
+				v.Min = v.Max;
+				v.Max = f;
 			}
 			variables.push_back (v);
 		}
+		X = i;
 		godata = go_data_vector_val_new (variables[i].Values, variables[i].NbValues, NULL);
 		gog_series_set_dim (series, 0, godata, NULL);
 		m_View->SetAxisBounds (GOG_AXIS_X, variables[i].Min, variables[i].Max, invert_axis);
 		m_View->SetAxisLabel (GOG_AXIS_X, _(UnitNames[variables[i].Unit]));
+		if (m_XAxisInvertBtn) {
+			g_signal_handler_block (m_XAxisInvertBtn, m_XAxisInvertSgn);
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (m_XAxisInvertBtn), invert_axis);
+			g_signal_handler_unblock (m_XAxisInvertBtn, m_XAxisInvertSgn);
+		}
 		if (integral > 0) {
 			g_object_ref (godata);
 		}
@@ -1624,22 +1797,171 @@ void SpectrumDocument::OnUnitChanged (int i)
 	}
 }
 
-double SpectrumDocument::GetConversionFactor (SpectrumUnitType oldu, SpectrumUnitType newu)
+void SpectrumDocument::OnYUnitChanged (int i)
 {
-	double res = go_nan;
-	switch (oldu) {
-	case GCU_SPECTRUM_UNIT_PPM:
-		if (go_finite (freq) && newu == GCU_SPECTRUM_UNIT_HZ)
-			res = freq;
-		break;
-	case GCU_SPECTRUM_UNIT_HZ:
-		if (go_finite (freq) && newu == GCU_SPECTRUM_UNIT_PPM)
-			res = 1. / freq;
+	SpectrumUnitType unit = GCU_SPECTRUM_UNIT_MAX;
+	bool invert_axis = false;
+	switch (m_SpectrumType) {
+	case GCU_SPECTRUM_INFRARED:
+	case GCU_SPECTRUM_RAMAN:
+	case GCU_SPECTRUM_UV_VISIBLE:
+		unit = (i == 0)? GCU_SPECTRUM_UNIT_ABSORBANCE: GCU_SPECTRUM_UNIT_TRANSMITTANCE;
 		break;
 	default:
 		break;
 	}
-	return res;
+	if (unit == GCU_SPECTRUM_UNIT_MAX)
+		return;
+	GOData *godata;
+	GogSeries *series = m_View->GetSeries ();
+	if (m_YUnit == unit) {
+		Y = -1;
+		godata = go_data_vector_val_new (y, npoints, NULL);
+		gog_series_set_dim (series, 1, godata, NULL);
+		m_View->SetAxisBounds (GOG_AXIS_Y, miny, maxy, invert_axis);
+		m_View->SetAxisLabel (GOG_AXIS_Y, _(UnitNames[m_YUnit]));
+	} else {
+		unsigned i, j;
+		double (*conv) (double, double, double);
+		double f, o;
+		for (i = 0; i < variables.size (); i++)
+			if (variables[i].Symbol == 'Y' && variables[i].Unit == unit)
+				break;
+		if (i == variables.size ()) {
+			// Add new data vector
+			JdxVar v;
+			if (Y >=0) {
+				conv = GetConversionFunction (variables[Y].Unit, unit, f, o);
+				if (!conv)
+					return;
+				v.Name = _(UnitNames[variables[Y].Unit]);
+				v.Symbol = variables[Y].Symbol;
+				v.Type = variables[Y].Type;
+				v.Unit = unit;
+				v.Format = variables[Y].Format;
+				v.NbValues = variables[Y].NbValues;
+				v.First = conv (variables[Y].First, f, o);
+				v.Last = conv (variables[Y].Last, f, o);
+				v.Min = conv (variables[Y].Min, f, o);
+				v.Max = conv (variables[Y].Max, f, o);
+				v.Factor = 1.;
+				v.Values = new double[variables[Y].NbValues];
+				for (j = 0; j < variables[Y].NbValues; j++)
+					v.Values[j] = conv (variables[Y].Values[j], f, o);
+			} else {
+				conv = GetConversionFunction (m_YUnit, unit, f, o);
+				if (!conv)
+					return;
+				v.Name = _(UnitNames[unit]);
+				v.Symbol = 'Y';
+				v.Type = GCU_SPECTRUM_TYPE_DEPENDENT;
+				v.Unit = unit;
+				v.Format = GCU_SPECTRUM_FORMAT_MAX;
+				v.NbValues = npoints;
+				v.First = conv (firsty, f, o);
+				v.Last = 0.; // not important
+				v.Min = conv (miny, f, o);
+				v.Max = conv (maxy, f, o);
+				v.Factor = 1.;
+				v.Values = new double[npoints];
+				for (j = 0; j < npoints; j++)
+					v.Values[j] = conv (y[j], f, o);
+			}
+			if (v.Min > v.Max) {
+				f = v.Min;
+				v.Min = v.Max;
+				v.Max = f;
+			}
+			variables.push_back (v);
+		}
+		Y = i;
+		godata = go_data_vector_val_new (variables[i].Values, variables[i].NbValues, NULL);
+		gog_series_set_dim (series, 1, godata, NULL);
+		m_View->SetAxisBounds (GOG_AXIS_Y, variables[i].Min, variables[i].Max, invert_axis);
+		m_View->SetAxisLabel (GOG_AXIS_Y, _(UnitNames[variables[i].Unit]));
+	}
+}
+
+static double mult (double val, double f, double offset)
+{
+	return val * f + offset;
+}
+
+static double inv (double val, double f, double offset)
+{
+	return f / val + offset;
+}
+
+static double logm (double val, double f, double offset)
+{
+	return -log10 (val * f + offset);
+}
+
+static double expm (double val, double f, double offset)
+{
+	return exp10 (-val) * f + offset;
+}
+
+double (*SpectrumDocument::GetConversionFunction (SpectrumUnitType oldu, SpectrumUnitType newu, double &factor, double &shift)) (double, double, double)
+{
+	switch (oldu) {
+	case GCU_SPECTRUM_UNIT_CM_1:
+		if (newu == GCU_SPECTRUM_UNIT_NANOMETERS) {
+			factor = 1.e7;
+			shift = 0;
+			return inv;
+		}
+		if (newu == GCU_SPECTRUM_UNIT_MICROMETERS) {
+			factor = 1.e4;
+			shift = 0;
+			return inv;
+		}
+		break;
+	case GCU_SPECTRUM_UNIT_TRANSMITTANCE:
+		if (newu == GCU_SPECTRUM_UNIT_ABSORBANCE) {
+			factor = 1.;
+			shift = 0.;
+			return logm;
+		}
+		break;
+	case GCU_SPECTRUM_UNIT_ABSORBANCE:
+		if (newu == GCU_SPECTRUM_UNIT_TRANSMITTANCE) {
+			factor = 1.;
+			shift = 0.;
+			return expm;
+		}
+		break;
+	case GCU_SPECTRUM_UNIT_PPM:
+		if (go_finite (freq) && newu == GCU_SPECTRUM_UNIT_HZ) {
+			factor = freq;
+			shift = 0;
+			return mult;
+		}
+		break;
+	case GCU_SPECTRUM_UNIT_NANOMETERS:
+		if (newu == GCU_SPECTRUM_UNIT_CM_1) {
+			factor = 1.e7;
+			shift = 0;
+			return inv;
+		}
+		break;
+	case GCU_SPECTRUM_UNIT_MICROMETERS:
+		if (newu == GCU_SPECTRUM_UNIT_CM_1) {
+			factor = 1.e4;
+			shift = 0;
+			return inv;
+		}
+		break;
+	case GCU_SPECTRUM_UNIT_HZ:
+		if (go_finite (freq) && newu == GCU_SPECTRUM_UNIT_PPM)
+			factor = 1. / freq;
+			shift = 0.;
+			return mult;
+		break;
+	default:
+		break;
+	}
+	return NULL;
 }
 
 void SpectrumDocument::OnShowIntegral ()
@@ -1755,7 +2077,7 @@ void SpectrumDocument::OnTransformFID (GtkButton *btn)
 		n <<= 1;
 	n <<= 1; // doubles the points number to get more transformed values
 	go_complex *fid = new go_complex[n], *sp;
-	unsigned i, j;
+	unsigned i;
 	for (i = 0; i < npoints; i++) {
 		// assuming we have as many real, imaginary and time values
 		fid[i].re = re[i];
@@ -1771,7 +2093,7 @@ void SpectrumDocument::OnTransformFID (GtkButton *btn)
 	// only the first half values are useful
 	n /= 2;
 	// copy the unphased data to Rt and It (t for transformed)
-	JdxVar vr, vi, rp, xf;
+	JdxVar vr, vi, rp;
 	vr.Name = _("Real transformed data");
 	vr.Symbol = 't';
 	vr.Type = GCU_SPECTRUM_TYPE_DEPENDENT;
@@ -1807,7 +2129,7 @@ void SpectrumDocument::OnTransformFID (GtkButton *btn)
 	It = variables.size ();
 	variables.push_back (vi);
 	// Now we need to adjust the phase (see http://www.ebyte.it/stan/Poster_EDISPA.html)
-	double phi = 0., tau = 0., q[36][31], *z;
+	double phi = 0., tau = 0., *z;
 	double step = M_PI * 2. * tau / n, phase;
 	z = new double[n];
 	for (i = 0; i < n; i++)
@@ -1839,28 +2161,18 @@ void SpectrumDocument::OnTransformFID (GtkButton *btn)
 		xo = x;
 		freq = (npoints - 1) / (lastx - firstx);
 	}
-	// first Hz
-	xf.Name = _(UnitNames[GCU_SPECTRUM_UNIT_HZ]);
-	xf.Symbol = 'X';
-	xf.Type = GCU_SPECTRUM_TYPE_INDEPENDENT;
-	xf.Unit = GCU_SPECTRUM_UNIT_HZ;
-	xf.Format = GCU_SPECTRUM_FORMAT_MAX;
-	xf.NbValues = n;
-	xf.Values = new double[n];
-	for (i = 0; i < n; i++)
-		xf.Values[i] = freq * (i + 1.) / n;
-	xf.First = xf.Min = xf.Values[0];
-	xf.Last = xf.Max = xf.Values[n - 1];
-	xf.Factor = 1.;
 	//now display the spectrum
 	variables[R].Series = NULL;
 	rp.Series = m_View->GetSeries ();
 	GOData *godata = go_data_vector_val_new (rp.Values, n, NULL);
 	gog_series_set_dim (rp.Series, 1, godata, NULL);
-	godata = go_data_vector_val_new (xf.Values, n, NULL);
+	godata = (X < 0)? go_data_vector_val_new (x, npoints, NULL): go_data_vector_val_new (variables[X].Values, variables[X].NbValues, NULL);
 	gog_series_set_dim (rp.Series, 0, godata, NULL);
-	m_View->SetAxisBounds (GOG_AXIS_X, xf.Min, xf.Max, true);
-	m_View->SetAxisLabel (GOG_AXIS_X, xf.Name.c_str ());
+}
+
+void SpectrumDocument::OnXAxisInvert (bool inverted)
+{
+	m_View->InvertAxis (GOG_AXIS_X, inverted);
 }
 
 }	//	nampespace gcu
