@@ -23,27 +23,16 @@
 #include "config.h"
 #include "spectrumdoc.h"
 #include "spectrumview.h"
-#ifndef GOFFICE_HAS_GLOBAL_HEADER
-#   include <goffice/data/go-data-simple.h>
-#   include <goffice/graph/gog-data-set.h>
-#   include <goffice/graph/gog-graph.h>
-#   include <goffice/graph/gog-label.h>
-#   include <goffice/graph/gog-object.h>
-#   include <goffice/graph/gog-plot.h>
-#   include <goffice/graph/gog-series.h>
-#   ifndef GOG_TYPE_LABEL
-#      define GOG_TYPE_LABEL GOG_LABEL_TYPE
-#      include <goffice/graph/gog-style.h>
-#      define GOStyle GogStyle
-#      define go_styled_object_get_style gog_styled_object_get_style
-#      define GO_STYLED_OBJECT GOG_STYLED_OBJECT
-#   else
-#      include <goffice/utils/go-style.h>
-#      include <goffice/utils/go-styled-object.h>
-#   endif
-#   include <goffice/gtk/go-graph-widget.h>
-#   include <goffice/utils/go-image.h>
-#endif
+#include <goffice/data/go-data-simple.h>
+#include <goffice/graph/gog-data-set.h>
+#include <goffice/graph/gog-graph.h>
+#include <goffice/graph/gog-label.h>
+#include <goffice/graph/gog-object.h>
+#include <goffice/graph/gog-plot.h>
+#include <goffice/graph/gog-series.h>
+#include <goffice/graph/gog-style.h>
+#include <goffice/gtk/go-graph-widget.h>
+#include <goffice/utils/go-image.h>
 #include <gsf/gsf-output-gio.h>
 #include <glib/gi18n-lib.h>
 #include <cmath>
@@ -65,10 +54,25 @@ static void on_max_changed (SpectrumView *view)
 	view->OnMaxChanged ();
 }
 
+static void on_ymin_changed (SpectrumView *view)
+{
+	view->OnYMinChanged ();
+}
+
+static void on_ymax_changed (SpectrumView *view)
+{
+	view->OnYMaxChanged();
+}
+
 static void on_xrange_changed (SpectrumView *view)
 {
 	view->OnXRangeChanged ();
 }
+
+static void on_yrange_changed (SpectrumView *view)
+{
+	view->OnYRangeChanged ();
+}
 	
 SpectrumView::SpectrumView (SpectrumDocument *pDoc)
 {
@@ -76,7 +80,7 @@ SpectrumView::SpectrumView (SpectrumDocument *pDoc)
 	m_Widget = go_graph_widget_new (NULL);
 	GogGraph *graph = go_graph_widget_get_graph (GO_GRAPH_WIDGET (m_Widget));
 	/* Add a title */
-	GogLabel *label = (GogLabel *) g_object_new (GOG_TYPE_LABEL, NULL);
+	GogLabel *label = (GogLabel *) g_object_new (GOG_LABEL_TYPE, NULL);
 	gog_object_add_by_name (GOG_OBJECT (graph), "Title", GOG_OBJECT (label));
 	/* Get the chart created by the widget initialization */
 	GogChart *chart = go_graph_widget_get_chart (GO_GRAPH_WIDGET (m_Widget));
@@ -103,6 +107,23 @@ SpectrumView::SpectrumView (SpectrumDocument *pDoc)
 	xrangesgn = g_signal_connect_swapped (xrange, "value-changed", G_CALLBACK (on_xrange_changed), this);
 	gtk_box_pack_start (GTK_BOX (box), GTK_WIDGET (xrange), true, true, 0);
 	gtk_box_pack_start (GTK_BOX (m_OptionBox), box, false, false, 0);
+	GtkWidget *box2 = gtk_hbox_new (false, 5);
+	w = gtk_label_new (_("Minimum Y value:"));
+	gtk_box_pack_start (GTK_BOX (box2), w, false, false, 0);
+	yminbtn = GTK_SPIN_BUTTON (gtk_spin_button_new_with_range (0., 1., 0.1));
+	yminsgn = g_signal_connect_swapped (yminbtn, "value-changed", G_CALLBACK (on_ymin_changed), this);
+	gtk_box_pack_start (GTK_BOX (box2), GTK_WIDGET (yminbtn), false, false, 0);
+	w = gtk_label_new (_("Maximum Y value:"));
+	gtk_box_pack_start (GTK_BOX (box2), w, false, false, 0);
+	ymaxbtn = GTK_SPIN_BUTTON (gtk_spin_button_new_with_range (0., 1., 0.1));
+	ymaxsgn = g_signal_connect_swapped (ymaxbtn, "value-changed", G_CALLBACK (on_ymax_changed), this);
+	gtk_box_pack_start (GTK_BOX (box2), GTK_WIDGET (ymaxbtn), false, false, 0);
+	yrange = GTK_RANGE (gtk_hscrollbar_new (NULL));
+	gtk_widget_set_sensitive (GTK_WIDGET (yrange), false);
+	yrangesgn = g_signal_connect_swapped (yrange, "value-changed", G_CALLBACK (on_yrange_changed), this);
+	gtk_box_pack_start (GTK_BOX (box2), GTK_WIDGET (yrange), true, true, 0);
+	gtk_box_pack_start (GTK_BOX (m_OptionBox), box2, false, false, 0);
+
 }
 
 SpectrumView::~SpectrumView ()
@@ -141,6 +162,31 @@ void SpectrumView::SetAxisBounds (GogAxisType target, double min, double max, bo
 		xmin = min;
 		xmax = max;
 	}
+	else if (target == GOG_AXIS_Y) {
+		double l = log10 (fabs (max - min));
+		int n = (l < 3)? rint (3 - l): 0;
+		ystep = pow (10., -n);
+		g_signal_handler_block (yminbtn, yminsgn); 
+		g_signal_handler_block (ymaxbtn, ymaxsgn); 
+		gtk_spin_button_set_range (yminbtn, min, max); 
+		gtk_spin_button_set_range (ymaxbtn, min, max); 
+		gtk_spin_button_set_increments (yminbtn, ystep, 100 * ystep); 
+		gtk_spin_button_set_increments (ymaxbtn, ystep, 100 * ystep);
+		gtk_range_set_increments (yrange, ystep, 100 * ystep);
+		gtk_range_set_inverted (yrange, !inverted);
+		g_signal_handler_block (yrange, yrangesgn);
+		gtk_range_set_value (yrange, 0.);
+		gtk_widget_set_sensitive (GTK_WIDGET (yrange), false);
+		g_signal_handler_unblock (yrange, yrangesgn);
+		gtk_spin_button_set_value (yminbtn, min);
+		gtk_spin_button_set_value (ymaxbtn, max);
+		gtk_spin_button_set_digits (yminbtn, n);
+		gtk_spin_button_set_digits (ymaxbtn, n);
+		g_signal_handler_unblock (yminbtn, yminsgn); 
+		g_signal_handler_unblock (ymaxbtn, ymaxsgn); 
+		ymin = min;
+		ymax = max;
+	}
 }
 
 void SpectrumView::SetAxisLabel (GogAxisType target, char const *unit)
@@ -155,7 +201,7 @@ void SpectrumView::SetAxisLabel (GogAxisType target, char const *unit)
 		gog_object_clear_parent (label);
 		g_object_unref (label);
 	}
-	label = GOG_OBJECT (g_object_new (GOG_TYPE_LABEL, NULL));
+	label = GOG_OBJECT (g_object_new (GOG_LABEL_TYPE, NULL));
 	gog_dataset_set_dim (GOG_DATASET (label), 0, data, NULL);
 	gog_object_add_by_name (axis, "Label", label);
 	
@@ -167,7 +213,7 @@ void SpectrumView::ShowAxis (GogAxisType target, bool show)
 	GSList *axes = gog_chart_get_axes (chart, target);
 	GogObject *axis = GOG_OBJECT (axes->data);
 	g_object_set (G_OBJECT (axis), "major-tick-labeled", false, NULL);
-	GOStyle *style = go_styled_object_get_style (GO_STYLED_OBJECT (axis));
+	GogStyle *style = gog_styled_object_get_style (GOG_STYLED_OBJECT (axis));
 	style->line.dash_type = GO_LINE_NONE;
 	style->line.auto_dash = false;
 }
@@ -205,6 +251,34 @@ void SpectrumView::OnMinChanged ()
 	g_signal_handler_unblock (xrange, xrangesgn);
 }
 
+void SpectrumView::OnYMinChanged ()
+{
+	double min = gtk_spin_button_get_value (yminbtn);
+	double max = gtk_spin_button_get_value (ymaxbtn);
+	if (max <= min) {
+		double step;
+		gtk_spin_button_get_increments (yminbtn, &step, NULL);
+		min = max - step;
+		g_signal_handler_block (yminbtn, yminsgn); 
+		gtk_spin_button_set_value (yminbtn, min);
+		g_signal_handler_unblock (yminbtn, yminsgn); 
+	}
+	GogChart *chart = go_graph_widget_get_chart (GO_GRAPH_WIDGET (m_Widget));
+	GSList *axes = gog_chart_get_axes (chart, GOG_AXIS_Y);
+	GogAxis *axis = GOG_AXIS (axes->data);
+	gog_axis_set_bounds (axis, min, max);
+	g_signal_handler_block (yrange, yrangesgn);
+	if (max - min < ymax - ymin) {
+		gtk_range_set_range (yrange, 0., ymax - ymin - max + min);
+		gtk_range_set_value (yrange, min - ymin);
+		gtk_widget_set_sensitive (GTK_WIDGET (yrange), true);
+	} else {
+		gtk_range_set_value (yrange, 0.);
+		gtk_widget_set_sensitive (GTK_WIDGET (yrange), false);
+	}
+	g_signal_handler_unblock (yrange, yrangesgn);
+}
+
 void SpectrumView::OnMaxChanged ()
 {
 	double min = gtk_spin_button_get_value (xminbtn);
@@ -233,6 +307,34 @@ void SpectrumView::OnMaxChanged ()
 	g_signal_handler_unblock (xrange, xrangesgn);
 }
 
+void SpectrumView::OnYMaxChanged ()
+{
+	double min = gtk_spin_button_get_value (yminbtn);
+	double max = gtk_spin_button_get_value (ymaxbtn);
+	if (max <= min) {
+		double step;
+		gtk_spin_button_get_increments (ymaxbtn, &step, NULL);
+		max = min + step;
+		g_signal_handler_block (ymaxbtn, ymaxsgn); 
+		gtk_spin_button_set_value (ymaxbtn, max);
+		g_signal_handler_unblock (ymaxbtn, ymaxsgn); 
+	}
+	GogChart *chart = go_graph_widget_get_chart (GO_GRAPH_WIDGET (m_Widget));
+	GSList *axes = gog_chart_get_axes (chart, GOG_AXIS_Y);
+	GogAxis *axis = GOG_AXIS (axes->data);
+	gog_axis_set_bounds (axis, min, max);
+	g_signal_handler_block (yrange, yrangesgn);
+	if (max - min < ymax - ymin) {
+		gtk_range_set_range (yrange, 0., ymax - ymin - max + min);
+		gtk_range_set_value (yrange, min - xmin);
+		gtk_widget_set_sensitive (GTK_WIDGET (yrange), true);
+	} else {
+		gtk_range_set_value (yrange, 0.);
+		gtk_widget_set_sensitive (GTK_WIDGET (yrange), false);
+	}
+	g_signal_handler_unblock (yrange, yrangesgn);
+}
+
 void SpectrumView::OnXRangeChanged ()
 {
 	double max = gtk_spin_button_get_value (xmaxbtn) - gtk_spin_button_get_value (xminbtn);
@@ -250,6 +352,23 @@ void SpectrumView::OnXRangeChanged ()
 	g_signal_handler_unblock (xmaxbtn, maxsgn); 
 }
 
+void SpectrumView::OnYRangeChanged ()
+{
+	double max = gtk_spin_button_get_value (ymaxbtn) - gtk_spin_button_get_value (yminbtn);
+	double min = ymin + gtk_range_get_value (yrange);
+	max += min;
+	GogChart *chart = go_graph_widget_get_chart (GO_GRAPH_WIDGET (m_Widget));
+	GSList *axes = gog_chart_get_axes (chart, GOG_AXIS_Y);
+	GogAxis *axis = GOG_AXIS (axes->data);
+	gog_axis_set_bounds (axis, min, max);
+	g_signal_handler_block (yminbtn, yminsgn); 
+	gtk_spin_button_set_value (yminbtn, min);
+	g_signal_handler_unblock (yminbtn, yminsgn); 
+	g_signal_handler_block (ymaxbtn, ymaxsgn);
+	gtk_spin_button_set_value (ymaxbtn, max);
+	g_signal_handler_unblock (ymaxbtn, ymaxsgn); 
+}
+
 GogSeries *SpectrumView::NewSeries (bool new_plot)
 {
 	GogChart *chart = go_graph_widget_get_chart (GO_GRAPH_WIDGET (m_Widget));
