@@ -4,7 +4,7 @@
  * GChemPaint text plugin
  * texttool.cc 
  *
- * Copyright (C) 2002-2010 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -25,35 +25,21 @@
 #include "config.h"
 #include "texttool.h"
 #include <gcp/application.h>
-#include <gcp/atom.h>
-#include <gcp/bond.h>
 #include <gcp/document.h>
 #include <gcp/fragment.h>
 #include <gcp/settings.h>
 #include <gcp/text.h>
 #include <gcp/theme.h>
+#include <gcp/view.h>
+#include <gcp/widgetdata.h>
 #include <gcp/window.h>
-#ifndef GOFFICE_HAS_GLOBAL_HEADER
-#	include <goffice/gtk/go-color-selector.h>
-#else
-#	include <gcu/goffice-compat.h>
-#endif
+#include <gccv/text.h>
+#include <goffice/gtk/go-color-selector.h>
 #include <gdk/gdkkeysyms.h>
 #include <unistd.h>
 #include <cstring>
 
 using namespace gcu;
-extern GtkTargetEntry const text_targets[];
-namespace gcp {
-extern xmlDocPtr pXmlDoc;
-};
-
-
-GtkTargetEntry const text_targets[] = {
-	{(char *) GCHEMPAINT_ATOM_NAME,  0, gcp::GCP_CLIPBOARD_NATIVE},
-	{(char *) "UTF8_STRING", 0, gcp::GCP_CLIPBOARD_UTF8_STRING},
-	{(char *) "STRING", 0, gcp::GCP_CLIPBOARD_STRING}
-};
 
 static void on_get_data (GtkClipboard *clipboard, GtkSelectionData *selection_data,  guint info, gcpTextTool* tool)
 {
@@ -118,7 +104,7 @@ gcpTextTool::gcpTextTool (gcp::Application* App, string Id):
 {
 	m_Active = NULL;
 	m_bUndo = true;
-	m_CurNode = m_InitNode = m_GroupNode = NULL;
+	m_CurNode = m_InitNode = NULL;
 	m_Strikethrough = false;
 	m_FontDesc = NULL;
 	gcp::Theme *pTheme = gcp::TheThemeManager.GetTheme ("Default");
@@ -132,7 +118,6 @@ gcpTextTool::gcpTextTool (gcp::Application* App, string Id):
 	m_Rise = 0;
 	m_Color = RGBA_BLACK;
 	m_SelSignal = 0;
-	m_Group = NULL;
 }
 
 gcpTextTool::~gcpTextTool ()
@@ -151,7 +136,7 @@ gcpTextTool::~gcpTextTool ()
 bool gcpTextTool::OnClicked ()
 {
 	if (m_Active && ((m_pObject == NULL) || (m_pObject->GetType () != TextType) ||
-			(m_Active != g_object_get_data (G_OBJECT (m_pData->Items[m_pObject]), "text"))))
+			(static_cast <gccv::Item *> (m_Active) != dynamic_cast <gccv::ItemClient *> (m_pObject)->GetItem ())))
 		Unselect ();
 	bool create = false;
 	if (!m_pObject) {
@@ -165,22 +150,19 @@ bool gcpTextTool::OnClicked ()
 	if (m_pObject) {
 		if (m_pObject->GetType () != TextType)
 			return false;
-		m_pObject->SetSelected (m_pWidget, gcp::SelStateUpdating);
-		m_Active = GNOME_CANVAS_PANGO (g_object_get_data (G_OBJECT (m_pData->Items[m_pObject]), "text"));
-		m_pView->SetGnomeCanvasPangoActive (m_Active);
-		g_object_set (G_OBJECT (m_Active), "editing", true, NULL);
+		static_cast <gcp::Text *> (m_pObject)->SetSelected (gcp::SelStateUpdating);
+		m_Active = static_cast <gccv::Text *> (dynamic_cast <gccv::ItemClient *> (m_pObject)->GetItem ());
+		m_pView->SetTextActive (m_Active);
+		m_Active->SetEditing (true);
 		m_CurNode = ((gcp::Text*) m_pObject)->SaveSelected ();
 		m_InitNode = ((gcp::Text*) m_pObject)->SaveSelected ();
 		m_pView->GetDoc ()->GetWindow ()->ActivateActionWidget ("/MainMenu/FileMenu/SaveAsImage", false);
-		if (m_SelSignal == 0)
-			m_SelSignal = g_signal_connect_swapped (m_Active, "sel-changed", G_CALLBACK (on_sel_changed), this);
+/*		if (m_SelSignal == 0)
+			m_SelSignal = g_signal_connect_swapped (m_Active, "sel-changed", G_CALLBACK (on_sel_changed), this);*/
 		if (create)
 			BuildAttributeList ();
 		else
 			UpdateAttributeList ();
-		m_Group = m_pObject->GetGroup ();
-		if (m_pView->GetDoc ()->GetCurrentOperation () == NULL && m_Group)
-			m_GroupNode = m_Group->Save (gcp::pXmlDoc);
 	}
 	return true;
 }
@@ -270,7 +252,7 @@ bool gcpTextTool::OnEvent (GdkEvent* event)
 					SetSizeFull (true);
 					return true;
 				case GDK_space: {
-					GnomeCanvasPango* saved = m_Active;
+					gccv::Text *saved = m_Active;
 					m_Active = NULL;
 					UpdateAttributeList ();
 					m_Active = saved;
@@ -288,9 +270,9 @@ bool gcpTextTool::OnEvent (GdkEvent* event)
 				((GdkEventKey*) event)->string = newstr;
 				((GdkEventKey*) event)->length = w;
 			}
-			gnome_canvas_item_grab_focus ((GnomeCanvasItem*) m_Active);
+/*			gnome_canvas_item_grab_focus ((GnomeCanvasItem*) m_Active);
 			GnomeCanvasItemClass* klass = GNOME_CANVAS_ITEM_CLASS (G_OBJECT_GET_CLASS (m_Active));
-			klass->event ((GnomeCanvasItem*) m_Active, event);
+			klass->event ((GnomeCanvasItem*) m_Active, event);*/
 			return true;
 		} else if (event->type == GDK_BUTTON_PRESS) {
 			 switch (event->button.button) {
@@ -298,8 +280,8 @@ bool gcpTextTool::OnEvent (GdkEvent* event)
 				return true;
 			}
 		} else if (event->type == GDK_MOTION_NOTIFY) {
-			GnomeCanvasItemClass* klass = GNOME_CANVAS_ITEM_CLASS (G_OBJECT_GET_CLASS (m_Active));
-			klass->event ((GnomeCanvasItem*) m_Active, event);
+/*			GnomeCanvasItemClass* klass = GNOME_CANVAS_ITEM_CLASS (G_OBJECT_GET_CLASS (m_Active));
+			klass->event ((GnomeCanvasItem*) m_Active, event);*/
 		}
 	}
 	return false;
@@ -331,11 +313,11 @@ bool gcpTextTool::Unselect ()
 		g_signal_handler_disconnect (m_Active, m_SelSignal);
 		m_SelSignal = 0;
 	}
-	g_object_set (G_OBJECT (m_Active), "editing", false, NULL);
-	m_pView->SetGnomeCanvasPangoActive (NULL);
-	Object *pObj = (Object*) g_object_get_data (G_OBJECT (m_Active), "object");
-	pObj->SetSelected (m_pWidget, gcp::SelStateUnselected);
-	char const *text = pango_layout_get_text (gnome_canvas_pango_get_layout (m_Active));
+	m_pView->SetTextActive (NULL);
+	m_Active->SetEditing (false);
+	m_Active->GetClient ()->SetSelected (gcp::SelStateUnselected);
+	Object *pObj = dynamic_cast <Object *> (m_Active->GetClient ());
+	char const *text = pango_layout_get_text (const_cast <PangoLayout *> (m_Active->GetLayout ()));
 	m_Active = NULL;
 	while (!m_UndoList.empty ()) {
 		xmlFree(m_UndoList.front ());
@@ -349,64 +331,34 @@ bool gcpTextTool::Unselect ()
 	xmlBufferPtr endbuf = xmlBufferCreate ();
 	xmlNodeDump (initbuf, m_pApp->GetXmlDoc (), m_InitNode, 0, 0);
 	xmlNodeDump (endbuf, m_pApp->GetXmlDoc (), m_CurNode, 0, 0);
-	gcp::Operation *pOp = NULL;
-	if (strcmp ((char*) initbuf->content, (char*) endbuf->content)) {
-		if (m_Group) {
-			pOp = m_pView->GetDoc ()->GetCurrentOperation ();
-			if (!pOp) {
+	if (strcmp ((char*) initbuf->content, (char*) endbuf->content))
+	{
+		char* initval = (char*) xmlNodeGetContent (m_InitNode);
+		char* endval = (char*) xmlNodeGetContent (m_CurNode);
+		gcp::Operation *pOp = NULL;
+		if ((initval && strlen (initval))) {
+			if (endval && strlen (endval)) {
 				pOp = m_pView->GetDoc ()->GetNewOperation (gcp::GCP_MODIFY_OPERATION);
-				pOp->AddNode (m_GroupNode, 0);
+				pOp->AddNode (m_InitNode, 0);
+				pOp->AddNode (m_CurNode, 1);
+				m_CurNode = m_InitNode = NULL;
+			} else {
+				pOp = m_pView->GetDoc ()->GetNewOperation (gcp::GCP_DELETE_OPERATION);
+				pOp->AddNode (m_InitNode);
+				m_InitNode = NULL;
 			}
-			pOp->AddNode (m_Group->Save (gcp::pXmlDoc), 1);
-			m_Group = NULL;
-			m_GroupNode = NULL;
-		} else {
-			char* endval = (char*) xmlNodeGetContent (m_CurNode);
-			char* initval = (char*) xmlNodeGetContent (m_InitNode);
-			gcp::Fragment *fragment = dynamic_cast <gcp::Fragment *> (pObj);
-			map<Atom*, Bond*>::iterator i;
-			gcp::Bond *pBond = (fragment)? reinterpret_cast <gcp::Bond *> (fragment->GetAtom ()->GetFirstBond (i)): NULL;
-			if ((initval && strlen (initval))) {
-				if (endval && strlen (endval)) {
-					pOp = m_pView->GetDoc ()->GetNewOperation (gcp::GCP_MODIFY_OPERATION);
-					pOp->AddNode (m_InitNode, 0);
-					pOp->AddNode (m_CurNode, 1);
-					if (pBond) {
-						pOp->AddObject (pBond, 0);
-						pOp->AddObject (pBond, 1);
-					}
-					m_CurNode = m_InitNode = NULL;
-				} else {
-					pOp = m_pView->GetDoc ()->GetNewOperation (gcp::GCP_DELETE_OPERATION);
-					pOp->AddNode (m_InitNode);
-					if (pBond)
-						pOp->AddObject (pBond);
-					m_InitNode = NULL;
-				}
-			} else if (endval && strlen (endval)) {
-				pOp = m_pView->GetDoc ()->GetNewOperation (gcp::GCP_ADD_OPERATION);
-				pOp->AddNode (m_CurNode);
-				m_CurNode = NULL;
-			}
-			if (initval)
-				xmlFree (initval);
-			if (endval)
-				xmlFree (endval);
+		} else if (endval && strlen (endval)) {
+			pOp = m_pView->GetDoc ()->GetNewOperation (gcp::GCP_ADD_OPERATION);
+			pOp->AddNode (m_CurNode);
+			m_CurNode = NULL;
 		}
+		if (initval)
+			xmlFree (initval);
+		if (endval)
+			xmlFree (endval);
 		if (pOp)
 			m_pView->GetDoc ()->PushOperation (pOp, m_bUndo);
 		m_bUndo = true;
-	} else {
-		if (m_Group) {
-			if (m_GroupNode) {
-				xmlFree (m_GroupNode);
-				m_GroupNode = NULL;
-			}
-			m_Group = NULL;
-		}
-		pOp = m_pView->GetDoc ()->GetCurrentOperation ();
-		if (pOp)
-			m_pView->GetDoc ()->AbortOperation ();
 	}
 	xmlBufferFree (initbuf);
 	xmlBufferFree (endbuf);
@@ -419,10 +371,8 @@ bool gcpTextTool::Unselect ()
 		Object* pMol = pObj->GetMolecule ();	//if pObj is a fragment
 		if (pMol)
 			pObj = pMol;
-		gcu::Object *parent = pObj->GetParent ();
 		m_pView->GetDoc ()->Remove (pObj);
 		m_pView->GetDoc ()->AbortOperation ();
-		parent->EmitSignal (gcp::OnChangedSignal);
 	}
 	m_pView->GetDoc ()->GetWindow ()->ActivateActionWidget ("/MainMenu/FileMenu/SaveAsImage", m_pView->GetDoc ()->HasChildren ());
 	return true;
@@ -438,9 +388,9 @@ bool gcpTextTool::DeleteSelection ()
 	if (!text)
 		return false;
 	text->GetSelectionBounds (start, end);
-	gcp_pango_layout_replace_text (gnome_canvas_pango_get_layout (m_Active),
+/*	gcp_pango_layout_replace_text (gnome_canvas_pango_get_layout (m_Active),
 												start, end - start, "", NULL);
-	gnome_canvas_pango_set_selection_bounds (m_Active, start, start);
+	gnome_canvas_pango_set_selection_bounds (m_Active, start, start);*/
 	text->OnChanged (true);
 	return true;
 }
@@ -466,7 +416,7 @@ bool gcpTextTool::CopySelection (GtkClipboard *clipboard)
 		xmlAddChild (pDoc->children, node);
 	else
 		return false;
-	gtk_clipboard_set_with_data (clipboard, text_targets, 3,
+	gtk_clipboard_set_with_data (clipboard, gcp::targets, gcp::ClipboardFormats,
 				(GtkClipboardGetFunc) on_get_data,
 				(GtkClipboardClearFunc) gcp::on_clear_data, this);
 	gtk_clipboard_request_contents (clipboard,
@@ -516,7 +466,7 @@ bool gcpTextTool::OnReceive (GtkClipboard *clipboard, GtkSelectionData *data, in
 	gcp::Text *text = (gcp::Text*) g_object_get_data (G_OBJECT (m_Active), "object");
 	unsigned start, end;
 	text->GetSelectionBounds (start, end);
-	PangoLayout *layout = gnome_canvas_pango_get_layout (m_Active);
+	PangoLayout *layout = NULL; // FIXME
 	switch (*DataType) {
 		case gcp::GCP_CLIPBOARD_NATIVE: {
 			xmlDocPtr xml = xmlParseMemory ((const char*) data->data, data->length);
@@ -527,7 +477,7 @@ bool gcpTextTool::OnReceive (GtkClipboard *clipboard, GtkSelectionData *data, in
 			}
 			node = node->children;
 			if (!strcmp ((char*) node->name, "text")) {
-				gcp_pango_layout_replace_text (layout, start, end - start, "", NULL);
+//				gcp_pango_layout_replace_text (layout, start, end - start, "", NULL);
 				text->LoadSelection (node, start);
 				xmlFreeDoc (xml);
 				return true; // otherwise, we'd call OnChange(true) twice.
@@ -543,7 +493,7 @@ bool gcpTextTool::OnReceive (GtkClipboard *clipboard, GtkSelectionData *data, in
 				pango_attr_list_insert (l, attr);
 				attr = pango_attr_size_new (pTheme->GetFontSize ());
 				pango_attr_list_insert (l, attr);
-				gcp_pango_layout_replace_text (layout, start, end - start, buf.c_str (), l);
+//				gcp_pango_layout_replace_text (layout, start, end - start, buf.c_str (), l);
 				pango_attr_list_unref (l);
 				l = fragment->GetAttrList ();
 				struct FragState s;
@@ -552,7 +502,7 @@ bool gcpTextTool::OnReceive (GtkClipboard *clipboard, GtkSelectionData *data, in
 				pango_attr_list_filter (l, (PangoAttrFilterFunc) filter_fragment, &s);
 				delete fragment;
 				start += buf.length ();
-				gnome_canvas_pango_set_selection_bounds (m_Active, start, start);
+//				gnome_canvas_pango_set_selection_bounds (m_Active, start, start);
 			} else {
 				xmlFreeDoc (xml);
 				return false;
@@ -562,7 +512,7 @@ bool gcpTextTool::OnReceive (GtkClipboard *clipboard, GtkSelectionData *data, in
 		}
 		case gcp::GCP_CLIPBOARD_UTF8_STRING: {
 			PangoAttrList *l = pango_attr_list_new ();
-			gcp_pango_layout_replace_text (layout, start, end - start, (char const *) data->data, l);
+//			gcp_pango_layout_replace_text (layout, start, end - start, (char const *) data->data, l);
 			pango_attr_list_unref (l);
 			break;
 		}
@@ -571,10 +521,10 @@ bool gcpTextTool::OnReceive (GtkClipboard *clipboard, GtkSelectionData *data, in
 			if (!g_utf8_validate ((const char*) data->data, data->length, NULL)) {
 				gsize r, w;
 				gchar* newstr = g_locale_to_utf8 ((const char*) data->data, data->length, &r, &w, NULL);
-				gcp_pango_layout_replace_text (layout, start, end - start, (char const *) data->data, l);
+//				gcp_pango_layout_replace_text (layout, start, end - start, (char const *) data->data, l);
 				g_free (newstr);
 			} else
-				gcp_pango_layout_replace_text (layout, start, end - start, (char const *) data->data, l);
+//				gcp_pango_layout_replace_text (layout, start, end - start, (char const *) data->data, l);
 			pango_attr_list_unref (l);
 			break;
 		}
@@ -616,7 +566,7 @@ bool gcpTextTool::OnUndo ()
 	tmp = (char*) xmlGetProp (node, (xmlChar*) "end-sel");
 	end = (int) strtoul (tmp, NULL, 10);
 	xmlFree (tmp);
-	gnome_canvas_pango_set_selection_bounds (m_Active, start, end);
+//	gnome_canvas_pango_set_selection_bounds (m_Active, start, end);
 	m_CurNode = node;
 	return true;
 }
@@ -641,7 +591,7 @@ bool gcpTextTool::OnRedo ()
 	tmp = (char*) xmlGetProp (node, (xmlChar*) "end-sel");
 	end = (int) strtoul (tmp, NULL, 10);
 	xmlFree (tmp);
-	gnome_canvas_pango_set_selection_bounds (m_Active, start, end);
+//	gnome_canvas_pango_set_selection_bounds (m_Active, start, end);
 	m_CurNode = node;
 	return true;
 }
@@ -663,18 +613,19 @@ void gcpTextTool::OnGetData (GtkClipboard *clipboard, GtkSelectionData *selectio
 {
 	xmlDocPtr pDoc = gcp::WidgetData::GetXmlDoc (clipboard);
 	guint *DataType = (clipboard == gtk_clipboard_get (GDK_SELECTION_CLIPBOARD))? &gcp::ClipboardDataType: &gcp::ClipboardDataType1;
+	if (gcp::ClipboardData) {
+		xmlFree (gcp::ClipboardData);
+	} 
 	*DataType = info;
 	gint size;
 	if (info) {
-		if (gcp::ClipboardTextData) {
-			g_free (gcp::ClipboardTextData);
-		} 
 		gcp::Text *text = new gcp::Text ();
 		text->Load (pDoc->children->children);
-		gcp::ClipboardTextData = g_strdup (text->GetBuffer ().c_str ());
+		PangoLayout *layout = text->GetLayout ();
+		gcp::ClipboardData = xmlStrdup ((xmlChar*) pango_layout_get_text (layout));
 		delete text;
-		size = strlen ((char*) gcp::ClipboardTextData);
-		gtk_selection_data_set_text (selection_data, (const gchar*) gcp::ClipboardTextData, size);
+		size = strlen ((char*) gcp::ClipboardData);
+		gtk_selection_data_set_text (selection_data, (const gchar*) gcp::ClipboardData, size);
 	} else {
 		xmlDocDumpFormatMemory (pDoc, &gcp::ClipboardData, &size, info);
 		gtk_selection_data_set (selection_data, gdk_atom_intern (GCHEMPAINT_ATOM_NAME, FALSE), 8,  (const guchar*) gcp::ClipboardData, size);
@@ -700,11 +651,11 @@ void gcpTextTool::UpdateAttributeList ()
 	m_Strikethrough = false;
 	m_Color = RGBA_BLACK;
 	if (m_Active) {
-		PangoLayout *layout;
+/*		PangoLayout *layout;
 		g_object_get (m_Active, "layout", &layout, NULL);
 		PangoAttrList *l = pango_layout_get_attributes (layout);
 		if (l)
-			pango_attr_list_filter (l, (PangoAttrFilterFunc) filter_attribute, this);
+			pango_attr_list_filter (l, (PangoAttrFilterFunc) filter_attribute, this);*/
 	}
 	// select the found face
 	GtkTreeIter iter;
@@ -752,7 +703,7 @@ void gcpTextTool::BuildAttributeList ()
 	pango_attr_list_insert (l, pango_attr_strikethrough_new (m_Strikethrough));
 	pango_attr_list_insert (l, pango_attr_rise_new (m_Rise));
 	pango_attr_list_insert (l, pango_attr_foreground_new (UINT_RGBA_R (m_Color) * 0x101, UINT_RGBA_G (m_Color) * 0x101, UINT_RGBA_B (m_Color) * 0x101));
-	gnome_canvas_pango_set_insert_attrs (m_Active, l);
+//	gnome_canvas_pango_set_insert_attrs (m_Active, l);
 	m_Dirty = false;
 	if (m_pView)
 		gtk_window_present (m_pView->GetDoc ()->GetWindow ()->GetWindow ());
@@ -984,7 +935,7 @@ void gcpTextTool::OnSelectFamily (GtkTreeSelection *selection)
 		pango_attr_list_insert (l, pango_attr_weight_new (m_Weight));
 		pango_attr_list_insert (l, pango_attr_stretch_new (m_Stretch));
 		pango_attr_list_insert (l, pango_attr_variant_new (m_Variant));
-		gnome_canvas_pango_apply_attrs_to_selection (m_Active, l);
+//		gnome_canvas_pango_apply_attrs_to_selection (m_Active, l);
 		pango_attr_list_unref (l);
 	}
 }
@@ -1011,7 +962,7 @@ void gcpTextTool::OnSelectFace (GtkTreeSelection *selection)
 		pango_attr_list_insert (l, pango_attr_weight_new (m_Weight));
 		pango_attr_list_insert (l, pango_attr_stretch_new (m_Stretch));
 		pango_attr_list_insert (l, pango_attr_variant_new (m_Variant));
-		gnome_canvas_pango_apply_attrs_to_selection (m_Active, l);
+//		gnome_canvas_pango_apply_attrs_to_selection (m_Active, l);
 		pango_attr_list_unref (l);
 	}
 }
@@ -1060,7 +1011,7 @@ void gcpTextTool::SetSizeFull (bool update_list)
 	if (m_Active) {
 		PangoAttrList *l = pango_attr_list_new ();
 		pango_attr_list_insert (l, pango_attr_size_new (m_Size));
-		gnome_canvas_pango_apply_attrs_to_selection (m_Active, l);
+//		gnome_canvas_pango_apply_attrs_to_selection (m_Active, l);
 		pango_attr_list_unref (l);
 	}
 }
@@ -1112,7 +1063,7 @@ void gcpTextTool::SelectBestFontFace ()
 
 int gcpTextTool::GetIndex ()
 {
-	return gnome_canvas_pango_get_cur_index (m_Active);
+//	return gnome_canvas_pango_get_cur_index (m_Active);
 }
 
 void gcpTextTool::OnUnderlineChanged (unsigned underline)
@@ -1122,7 +1073,7 @@ void gcpTextTool::OnUnderlineChanged (unsigned underline)
 	if (m_Active) {
 		PangoAttrList *l = pango_attr_list_new ();
 		pango_attr_list_insert (l, pango_attr_underline_new (m_Underline));
-		gnome_canvas_pango_apply_attrs_to_selection (m_Active, l);
+//		gnome_canvas_pango_apply_attrs_to_selection (m_Active, l);
 		pango_attr_list_unref (l);
 	}
 }
@@ -1134,7 +1085,7 @@ void gcpTextTool::OnStriketroughToggled (bool strikethrough)
 	if (m_Active) {
 		PangoAttrList *l = pango_attr_list_new ();
 		pango_attr_list_insert (l, pango_attr_strikethrough_new (m_Strikethrough));
-		gnome_canvas_pango_apply_attrs_to_selection (m_Active, l);
+//		gnome_canvas_pango_apply_attrs_to_selection (m_Active, l);
 		pango_attr_list_unref (l);
 	}
 }
@@ -1152,7 +1103,7 @@ void gcpTextTool::OnForeColorChanged (GOColor color)
 	if (m_Active) {
 		PangoAttrList *l = pango_attr_list_new ();
 		pango_attr_list_insert (l, pango_attr_foreground_new (UINT_RGBA_R (m_Color) * 0x101, UINT_RGBA_G (m_Color) * 0x101, UINT_RGBA_B (m_Color) * 0x101));
-		gnome_canvas_pango_apply_attrs_to_selection (m_Active, l);
+//		gnome_canvas_pango_apply_attrs_to_selection (m_Active, l);
 		pango_attr_list_unref (l);
 	}
 }
