@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
  */
 
@@ -360,7 +360,7 @@ void GChemTableApp::OnAbout ()
 		"GNU General Public License for more details.\n\n"
 		"You should have received a copy of the GNU General Public License\n"
 		"along with this program; if not, write to the Free Software\n"
-		"Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301\n"
+		"Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307\n"
 		"USA";
 	
 	gtk_about_dialog_set_url_hook (on_about_activate_url, this, NULL);
@@ -656,6 +656,7 @@ void GChemTableApp::OnSaveAsImage (GChemTableCurve *curve)
 	if (!curve)
 		return;
 	list<string> l;
+	unsigned n = 0;
 	char const *mime;
 	map<string, GdkPixbufFormat*>::iterator i, end = m_SupportedPixbufFormats.end ();
 	for (i = m_SupportedPixbufFormats.begin (); i != end; i++)
